#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2019-2022 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################

import argparse
import subprocess
import os
import re # regexp package
import sys
import tempfile
import json
import xml.etree.ElementTree as ET




def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = "store_true")
    parser.add_argument('-t', '--tol',         required=True, default = 2.0,type=float)
    user_args, unknown_args = parser.parse_known_args()

    verbose=user_args.verbose
    percentage_tol = user_args.tol
    data = []
    num_files = len(unknown_args)

    titles = []
    for file_index in range(num_files):
        titles.append(os.path.basename(os.path.splitext(unknown_args[file_index])[0]))

    for file_index in range(num_files):
        with open(unknown_args[file_index],"r") as f:
            data.append(json.load(f))

    cmd = [d['cmdline'] for d in data]
    xargs = [d['xargs'] for d in data]
    yargs = [d['yargs'] for d in data]
    samples = [d['results'] for d in data]
    num_samples = len(samples[0])
    len_xargs = len(xargs[0])

    if verbose:
        print('//rocsparse-bench-regression CONFIG')
        for i in range(num_files):
            print('//rocsparse-bench-regression file'+str(i) +'      : \'' + unknown_args[i] + '\'')
        print('//rocsparse-bench-regression COMPARISON')

####
    for i in range(1,num_files):
        if xargs[0] != xargs[i]:
            print('xargs\'s must be equal, xargs from file \''+unknown_args[i]+'\' is not equal to xargs from file \''+unknown_args[0]+'\'')
            exit(1)

    if verbose:
        print('//rocsparse-bench-regression  -  xargs checked.')
####
    for i in range(1,num_files):
        if yargs[0] != yargs[i]:
            print('yargs\'s must be equal, yargs from file \''+unknown_args[i]+'\' is not equal to yargs from file \''+unknown_args[0]+'\'')
            exit(1)
    if verbose:
        print('//rocsparse-bench-regression  -  yargs checked.')
####
    for i in range(1,num_files):
        if num_samples != len(samples[i]):
            print('num_samples\'s must be equal, num_samples from file \''+unknown_args[i]+'\' is not equal to num_samples from file \''+unknown_args[0]+'\'')
            exit(1)
    if verbose:
        print('//rocsparse-bench-regression  -  num samples checked.')
####
    if verbose:
        print('//rocsparse-bench-regression percentage_tol: ' + str(percentage_tol) + '%')

    global_regression=False
    for file_index in range(1,num_files):
        print("//rocsparse-bench-regression - '"+ unknown_args[file_index])
        for iplot in range(len(yargs[0])):
            print('//rocsparse-bench-regression plot index ' + str(iplot) + ': \'' + yargs[0][iplot] + '\'',end='')
            mx_rel_flops=0
            mx_rel_time=0
            mx_rel_bandwidth=0
            mn_rel_flops=0
            mn_rel_time=0
            mn_rel_bandwidth=0
            regression=False
            for ixarg  in range(len_xargs):
                isample = iplot * len_xargs + ixarg
                tg = samples[file_index][isample]["timing"]
                tg0=samples[0][isample]["timing"]
                rel_flops = 100*(float(tg["flops"][0])-float(tg0["flops"][0]))/float(tg0["flops"][0])

                rel_time = 100*(float(tg["time"][0])-float(tg0["time"][0]))/float(tg0["time"][0])
                rel_bandwidth = 100*(float(tg["bandwidth"][0])-float(tg0["bandwidth"][0]))/float(tg0["bandwidth"][0])

                if ixarg > 0:
                    mx_rel_flops=max(mx_rel_flops,rel_flops)
                    mn_rel_flops=min(mn_rel_flops,rel_flops)

                    mx_rel_time=max(mx_rel_time,rel_time)
                    mn_rel_time=min(mn_rel_time,rel_time)

                    mx_rel_bandwidth=max(mx_rel_bandwidth,rel_bandwidth)
                    mn_rel_bandwidth=min(mn_rel_bandwidth,rel_bandwidth)
                else:
                    mx_rel_flops=rel_flops
                    mn_rel_flops=rel_flops

                    mx_rel_time=rel_time
                    mn_rel_time=rel_time

                    mx_rel_bandwidth=rel_bandwidth
                    mn_rel_bandwidth=rel_bandwidth

                if (rel_flops < -percentage_tol) or (rel_time < -percentage_tol) or (rel_bandwidth < -percentage_tol):
                    print("")
                if (rel_flops < -percentage_tol):
                    regression=True
                    print("//rocsparse-bench-regression   FAIL flops exceeds tolerance of  "  +  str(percentage_tol) + "%, [" +"{:.2f}".format(mn_rel_flops) + "," + "{:.2f}".format(mx_rel_flops) + "] from '" + xargs[file_index][ixarg] + "'")
                if (rel_time < -percentage_tol):
                    regression=True
                    print("//rocsparse-bench-regression   FAIL time exceeds tolerance of  "  +  str(percentage_tol) + "%, [" +"{:.2f}".format(mn_rel_time) + "," + "{:.2f}".format(mx_rel_time) + "] from '" + xargs[file_index][ixarg] + "'")
                if (rel_bandwidth < -percentage_tol):
                    regression=True
                    print("//rocsparse-bench-regression   FAIL bandwidth exceeds tolerance of  "  +  str(percentage_tol) + "%, [" +"{:.2f}".format(mn_rel_bandwidth) + "," + "{:.2f}".format(mx_rel_bandwidth) + "] from '" + xargs[file_index][ixarg] + "'")

            if regression:
                global_regression=True
            if regression:
                print('//rocsparse-bench-regression plot index ' + str(iplot) + ': \'' + yargs[0][iplot] + '\' FAILED')
            else:
                print("   PASSED")
        if verbose:
            print("//rocsparse-bench-regression    flops [" +"{:.2f}".format(mn_rel_flops) + "," + "{:.2f}".format(mx_rel_flops) + "], " + "time [" +"{:.2f}".format(mn_rel_time) + "," + "{:.2f}".format(mx_rel_time) + "], " + "bandwidth [" +"{:.2f}".format(mn_rel_bandwidth) + "," + "{:.2f}".format(mx_rel_bandwidth) + "]")
    if global_regression:
        exit(1)
    else:
        exit(0)
if __name__ == "__main__":
    main()




#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2021 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################

import argparse
import json
import os
import re
import subprocess

#
#
# This script filters the .json input file and run cpp-check.
# It is expected to use a version of cpp-check offering googleconfig.cfg.
#
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--ifilename',     required=False, default = 'compile_commands.json',
                        help = '.json file format is required.')
    parser.add_argument('-o', '--ofilename',     required=False, default = 'a.xml',
                        help = 'output file name.')
    parser.add_argument('-e', '--exclude',     required=False, default = '',
                        help = 'Regular expression to Exclude files whom names match with')
    parser.add_argument('-i', '--include',     required=False, default = '',
                        help = 'Regular expression to Include files whom names match with')
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = 'store_true', help = 'enable verbose')

    user_args, unknown_args = parser.parse_known_args()
    verbose=user_args.verbose
    exclude=user_args.exclude
    include=user_args.include
    ifilename=user_args.ifilename
    ofilename=user_args.ofilename

    ifilesplit = os.path.splitext(os.path.basename(ifilename))
    ofilesplit = os.path.splitext(os.path.basename(ofilename))

    ibasename=ifilesplit[0]
    iextension=ifilesplit[1]
    obasename=ofilesplit[0]
    oextension=ofilesplit[1]

    if verbose:
        print('// rocsparse-cppcheck.verbose:file \'' + ifilename + '\'')
    if len(unknown_args) > 0:
        print('// rocsparse-cppcheck.verbose:unexpected arguments ',end='')
        print(unknown_args)
        exit(1)
    with open(ifilename,'r') as f:
        compile_commands=json.load(f)

    res_inc=[ i for i in compile_commands if re.search(include,i['file']) != None]
    if exclude != '':
        compile_commands=[ i for i in res_inc if re.search(exclude,i['file']) == None]
    else:
        compile_commands = res_inc

    if verbose:
        print('// rocsparse-cppcheck.verbose:num commands ' + str(len(compile_commands)))

    tempfilename=ibasename + '.tmp.json'
    with open(tempfilename,'w') as f:
        json.dump(compile_commands,f)

    if verbose:
        print('// rocsparse-cppcheck.verbose:' + ofilename)
    cmdcppcheck = ['cppcheck',
                   '--enable=all',
                   '--project='+ tempfilename,
                   '--output-file='+ ofilename,
                   '--library=googletest.cfg',
                   '--suppress=missingIncludeSystem',
                   '--suppress=unusedFunction',
                   '--suppress=cstyleCast',
                   '--suppress=constParameter',
                   '--suppress=templateRecursion',
    #              '--suppress=invalidPointerCast'
                   '--suppress=unmatchedSuppression']

    if oextension == '.xml':
        cmdcppcheck.append('--xml')
    proc = subprocess.Popen(cmdcppcheck)
    proc.wait()
    rc = proc.returncode
    if rc != 0:
        print('// cppcheck failed')
        exit(1)

    if not verbose:
        os.remove(tempfilename)

if __name__ == "__main__":
    main()




#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################


import argparse
import json

##
def export_csv(obasename, delim, legend, results, verbose = False,debug = False):
    if verbose:
        print('//rocsparse-memstat  - output file : \'' + obasename + '\'')
    out = open(obasename, "w+")
    for i in range(len(results)):
        for j in range(len(legend)):
            field = results[i][legend[j]]
            if (j>0):
                if (legend[j]=="tag"):
                    out.write(delim+"\""+field+"\"")
                else:
                    out.write(delim+field)
            else:
                if (legend[j]=="tag"):
                    out.write("\""+field+"\"")
                else:
                    out.write(field)
        out.write('\n')
    out.close()

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-w', '--workingdir',     required=False, default = './')
    parser.add_argument('-o', '--obasename',    required=False, default = 'a.csv')
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = "store_true")
    parser.add_argument('-d', '--debug',         required=False, default = False, action = "store_true")
    user_args, unknown_args = parser.parse_known_args()
    verbose=user_args.verbose
    debug=user_args.debug
    obasename = user_args.obasename
    if len(unknown_args) > 1:
        print('expecting only one input file.')
    with open(unknown_args[0],"r") as f:
        case=json.load(f)

    results = case['results']
    legend =  case['legend']
    leaks=case['leaks']
    if (len(leaks)==0):
        print('//rocsparse-memstat state: clean.')
    else:
        print('//rocsparse-memstat state: unclean, leaking:')
        for j in range(len(leaks)):
            print(f"mode: {leaks[j]['mode']}",end="")
            print(f", size: {leaks[j]['nbytes']} bytes",end="")
            print(f", location: {leaks[j]['tag']}")
    if verbose:
        print('//rocsparse-memstat  - input file :  \'' + unknown_args[0] + '\'')

    export_csv( obasename,', ', legend, results, verbose,debug)

if __name__ == "__main__":
    main()




#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################


import argparse
import subprocess
import os
import re # regexp package
import sys
import tempfile
import json


def defcurves(out,ofilename,title,ifilename,x_label, y_label, titles):
    nplots = len(titles)
    out.write("reset\n")
    out.write("set grid\n")
    out.write("set term pdfcairo enhanced color font 'Helvetica,9'\n")
    out.write("set output \"" + ofilename + "\"\n")
    out.write("set termoption noenhanced\n")
    out.write("set tmargin 0\n")
    out.write("set bmargin 0\n")
    out.write("set xlabel \"" + x_label + "\"\n")
    out.write("set ylabel \"" + y_label + "\"\n")
#    out.write("set logscale x\n")
    out.write("set logscale y\n")
    out.write("set offsets 0.25, 0.25, 0, 0\n")
    out.write("set xtics rotate by -45\n")
    out.write("set size ratio 0.35\n")
    out.write("set style fill noborder\n")
    out.write("set title '" + title +"'\n")
    out.write("plot '"+ifilename+"' using 1:2 with lines title '"+ titles[0] +"'")
    out.write(", '' "+" using 1:3 with lines title '"+titles[1]+"'")
    out.write(", '' "+" using 1:4 with lines title '"+titles[2]+"'")
#    out.write("plot '"+ifilename+"' using 1:"+str(2)+" with lines title '"+ titles[0] +"'")
#    out.write(", '' "+" using 1:" + str(3) + " with lines title '"+titles[1]+"'")
#    for i in range(len(titles)):
#
#            if (i>0):
#            out.write(",\\\n '' "+" using 1:" + str(i+1) + " with linespoints title '"+titles[i]+"'")
#        else:
#            out.write("plot '"+ifilename+" using 1:"+str(2)+" with linespoints title '"+ titles[0] +"'")
#        out.write("\n")



import rocsparse_bench_gnuplot_helper

#
# EXPORT TO PDF WITH GNUPLOT
#
##
def export_gnuplot(obasename, legend, results, verbose = False,debug = False):

    datafile = open(obasename + ".dat", "w+")
    for i in range(len(results)):
        tg = results[i]
        datafile.write(str(float(tg["time"])/float(results[len(results)-1]["time"])) + " " +
                       str(float(tg["nbytes_host"])/1024/1024) + " " + str(float(tg["nbytes_device"])/1024/1024) +  " " + str(float(tg["nbytes_managed"])/1024/1024) + "\n")
    datafile.close();

    if verbose:
        print('//rocsparse-memstat-plot  -  write gnuplot file : \'' + obasename + '.gnuplot\'')

    cmdfile = open(obasename + ".gnuplot", "w+")
    # for each plot
    num_curves=3
    filetype="pdf"
    filename_extension= "." + filetype

    defcurves(cmdfile,
              obasename + filename_extension,
              'Memory',
              obasename + ".dat",
              "operations",
              "Mbytes",
              ["host","device","managed"])
    cmdfile.close();

    rocsparse_bench_gnuplot_helper.call(obasename + ".gnuplot")
    if verbose:
        print('//rocsparse-memstat-plot CLEANING')

    if not debug:
        os.remove(obasename + '.dat')
        os.remove(obasename + '.gnuplot')

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-w', '--workingdir',     required=False, default = './')
    parser.add_argument('-o', '--obasename',    required=False, default = 'a')
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = "store_true")
    parser.add_argument('-d', '--debug',         required=False, default = False, action = "store_true")
    user_args, unknown_args = parser.parse_known_args()
    verbose=user_args.verbose
    debug=user_args.debug
    obasename = user_args.obasename
    if len(unknown_args) > 1:
        print('expecting only one input file.')
    with open(unknown_args[0],"r") as f:
        case=json.load(f)

    results = case['results']
    legend =  case['legend']
    if verbose:
        print('//rocsparse-memstat-plot')
        print('//rocsparse-memstat-plot  - file : \'' + unknown_args[0] + '\'')

    export_gnuplot( obasename, legend, results, verbose,debug)

if __name__ == "__main__":
    main()




#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2023 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################

import argparse
import os
import json
import xml.etree.ElementTree as ET
import rocsparse_bench_gnuplot_helper

def multiplot(out,ofilename,title,curve_indeces,plot_indeces,curves_per_plot,ifilename,y_range,y_label,col_index,curve_titles,plot_titles,xtic_label,linear=False):
    ncurves = len(curve_indeces)
    nplots = len(plot_indeces)

    if (ncurves == 0) or (nplots == 0):
        return

    xtic_label = xtic_label if len(xtic_label) != 0 else '1'

    out.write("reset\n")
    out.write("set term pdfcairo enhanced color font 'Helvetica,5'\n")
    out.write("set output \"" + ofilename + "\"\n")
    out.write("set termoption noenhanced\n")

    out.write("set ylabel \"" + y_label + "\"\n")

    cols = min(nplots, 3)
    rows = min(int((nplots - 1) / cols + 1), 2)

    plots_per_page = cols*rows
    pages = int((nplots - 1) / (plots_per_page) + 1)

    for p in range(pages):
        out.write("set multiplot title \"" + title + "\" font 'Helvetica,9' layout " + str(rows) + "," + str(cols) + " scale 1,1\n")

        if not linear:
            out.write("set logscale y\n")

        for i in range(plots_per_page):
            plot_index = p * plots_per_page + i

            if plot_index >= nplots:
                break

            if(plot_index % cols == 0):
                out.write("set ylabel \"" + y_label + "\"\n")
            else:
                out.write("set ylabel \" \"\n")

            out.write("set yrange [" + str(y_range[0]) + ":" + str(y_range[1]) + "]\n")
            out.write("set xtics rotate by -45\n")

            out.write("set title \"" + plot_titles[plot_index] + "\"\n")

            index = plot_index * curves_per_plot + curve_indeces[0]
            out.write("plot '"+ifilename+"' index "+str(index)+" using 1:"+str(col_index)+":xtic("+xtic_label+") with lines title '"+ curve_titles[0] +"'")
            for j in range(1,ncurves):
                index = plot_index * curves_per_plot + curve_indeces[j]
                out.write(",\\\n '' index "+str(index)+" using 1:"+str(col_index) + ":xtic("+xtic_label+") with lines title '"+curve_titles[j]+"'")

            out.write("\n")
        out.write("unset multiplot\n")

def export_gnuplot(obasename,xargs, yargs, case_results,case_titles,xtic_label = '',verbose = False,debug = False,linear=False):
    num_cases = len(case_results)
    datafile = open(obasename + ".dat", "w+")
    len_xargs = len(xargs)

    min_allowed_yvalue = 0.0
    max_time = float(case_results[0][0]["timing"]["time"][2])
    max_flops = float(case_results[0][0]["timing"]["flops"][2])
    max_bandwidth = float(case_results[0][0]["timing"]["bandwidth"][2])
    min_time = float(case_results[0][0]["timing"]["time"][0])
    min_flops = float(case_results[0][0]["timing"]["flops"][0])
    min_bandwidth = float(case_results[0][0]["timing"]["bandwidth"][0])

    for iplot in range(len(yargs)):
        for case_index in range(num_cases):
            samples = case_results[case_index]
            for ixarg  in range(len_xargs):
                isample = iplot * len_xargs + ixarg
                tg = samples[isample]["timing"]

                min_allowed_yvalue = 0.0
                if not linear:
                    min_allowed_yvalue = 1.0

                time0 = max(float(tg["time"][0]), min_allowed_yvalue)
                time1 = max(float(tg["time"][1]), min_allowed_yvalue)
                time2 = max(float(tg["time"][2]), min_allowed_yvalue)
                flops0 = max(float(tg["flops"][0]), min_allowed_yvalue)
                flops1 = max(float(tg["flops"][1]), min_allowed_yvalue)
                flops2 = max(float(tg["flops"][2]), min_allowed_yvalue)
                bandwidth0 = max(float(tg["bandwidth"][0]), min_allowed_yvalue)
                bandwidth1 = max(float(tg["bandwidth"][1]), min_allowed_yvalue)
                bandwidth2 = max(float(tg["bandwidth"][2]), min_allowed_yvalue)

                max_time = max(max_time, time2)
                max_flops = max(max_flops, flops2)
                max_bandwidth = max(max_bandwidth, bandwidth2)
                min_time = min(min_time, time0)
                min_flops = min(min_flops, flops0)
                min_bandwidth = min(min_bandwidth, bandwidth0)

                datafile.write(os.path.basename(os.path.splitext(xargs[ixarg])[0]) + " " +
                               str(time0) + " " +
                               str(time1) + " " +
                               str(time2) + " " +
                               str(flops0) + " " +
                               str(flops1) + " " +
                               str(flops2) + " " +
                               str(bandwidth0) + " " +
                               str(bandwidth1) + " "+
                               str(bandwidth2) + " " +
                               "\n")
            datafile.write("\n")
            datafile.write("\n")
    datafile.close();

    # pad y-axis
    pad_ratio = 0.05
    min_time = (1 + pad_ratio) * min_time - pad_ratio * max_time
    min_flops = (1 + pad_ratio) * min_flops - pad_ratio * max_flops
    min_bandwidth = (1 + pad_ratio) * min_bandwidth - pad_ratio * max_bandwidth

    max_time = (1 + pad_ratio) * max_time - pad_ratio * min_time
    max_flops = (1 + pad_ratio) * max_flops - pad_ratio * min_flops
    max_bandwidth = (1 + pad_ratio) * max_bandwidth - pad_ratio * min_bandwidth

    # fix y-axis for log scale
    if not linear:
        min_time = max(min_time, 1)
        min_flops = max(min_flops, 1)
        min_bandwidth = max(min_bandwidth, 1)

    if verbose:
        print('//rocsparse-bench-multiplot  -  write gnuplot file : \'' + obasename + '.gnuplot\'')

    cmdfile = open(obasename + ".gnuplot", "w+")
    # for each plot
    num_plots=len(yargs)
    filetype="pdf"
    filename_extension= "." + filetype

    multiplot(cmdfile,
              obasename + "_msec"+ filename_extension,
              'Time',
              range(num_cases),
              range(num_plots),
              num_cases,
              obasename + ".dat",
              [min_time, max_time],
              "milliseconds",
              2,
              case_titles,
              yargs, xtic_label, linear)
    multiplot(cmdfile,
              obasename + "_gflops"+ filename_extension,
              'Performance',
              range(num_cases),
              range(num_plots),
              num_cases,
              obasename + ".dat",
              [min_flops, max_flops],
              "GFlops",
              5,
              case_titles,
              yargs, xtic_label, linear)
    multiplot(cmdfile,
              obasename + "_bandwidth"+ filename_extension,
              'Bandwidth',
              range(num_cases),
              range(num_plots),
              num_cases,
              obasename + ".dat",
              [min_bandwidth, max_bandwidth],
              "GBytes/s",
              8,
              case_titles,
              yargs, xtic_label, linear)
    cmdfile.close();

    rocsparse_bench_gnuplot_helper.call(obasename + ".gnuplot")
    if verbose:
        print('//rocsparse-bench-multiplot CLEANING')

    if not debug:
        os.remove(obasename + '.dat')
        os.remove(obasename + '.gnuplot')


#
#
# MAIN
#
#
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', '--obasename',    required=False, default = 'a')
    parser.add_argument('-c', '--casenames',    required=False, default = "", help='name of curves separated by \';\'. e.g., case1;case2')
    parser.add_argument('-y', '--yargnames',    required=False, default = "", help='plot titles separated by \';\'. e.g., plot1;plot2')
    parser.add_argument('-t', '--xticlabel',    required=False, default = "", help='formula for xtic()')
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = "store_true")
    parser.add_argument('-d', '--debug',         required=False, default = False, action = "store_true")
    parser.add_argument('-l', '--linear',         required=False, default = False, action = "store_true")
    user_args, case_names = parser.parse_known_args()

    verbose=user_args.verbose
    debug=user_args.debug
    obasename = user_args.obasename
    linear = user_args.linear

    cases = []
    num_cases = len(case_names)

    case_titles = []
    for case_index in range(num_cases):
        case_titles.append(os.path.basename(os.path.splitext(case_names[case_index])[0]))

    for case_index in range(num_cases):
        with open(case_names[case_index],"r") as f:
            cases.append(json.load(f))

    cmd = [case['cmdline'] for case in cases]
    xargs = [case['xargs'] for case in cases]
    yargs = [case['yargs'] for case in cases]
    case_results = [case['results'] for case in cases]
    num_samples = len(case_results[0])

    case_names = user_args.casenames.split(";") if len(user_args.casenames) != 0 else case_titles
    yarg_names = user_args.yargnames.split(";") if len(user_args.yargnames) != 0 else yargs[0]

    if verbose:
        print('//rocsparse-bench-multiplot INPUT CASES')
        for case_index in range(num_cases):
            print('//rocsparse-bench-multiplot  - case'+str(case_index) +'      : \'' + case_names[case_index] + '\'')
        print('//rocsparse-bench-multiplot CHECKING')

####
    for case_index in range(1,num_cases):
        if xargs[0] != xargs[case_index]:
            print('xargs\'s must be equal, xargs from case \''+case_names[case_index]+'\' is not equal to xargs from case \''+case_names[0]+'\'')
            exit(1)

    if verbose:
        print('//rocsparse-bench-multiplot  -  xargs checked.')
####
    for case_index in range(1,num_cases):
        if yargs[0] != yargs[case_index]:
            print('yargs\'s must be equal, yargs from case \''+case_names[case_index]+'\' is not equal to yargs from case \''+case_names[0]+'\'')
            exit(1)
    if verbose:
        print('//rocsparse-bench-multiplot  -  yargs checked.')

    if len(yargs[0]) != len(yarg_names):
        print('yargnames length does not equal yargs length')
        exit(1)
    if verbose:
        print('//rocsparse-bench-multiplot  -  yargsnames checked.')
####
    for case_index in range(1,num_cases):
        if num_samples != len(case_results[case_index]):
            print('num_samples\'s must be equal, num_samples from case \''+case_names[case_index]+'\' is not equal to num_samples from case \''+case_names[0]+'\'')
            exit(1)
    if verbose:
        print('//rocsparse-bench-multiplot  -  num samples checked.')

    if num_cases != len(case_names):
        print('num_cases does not equal case_names length')
        exit(1)
    if verbose:
        print('//rocsparse-bench-multiplot  -  casenames checked.')
####
    if verbose:
        print('//rocsparse-bench-multiplot  -  write data    file : \'' + obasename + '.dat\'')

    export_gnuplot(obasename,
                   xargs[0],
                   yarg_names,
                   case_results,
                   case_names,
                   user_args.xticlabel,
                   verbose,
                   debug,
                   linear)

if __name__ == "__main__":
    main()




#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################

"""Expand rocSPARSE YAML test data file into binary Arguments records"""

import re
import sys
import os
import argparse
import ctypes
import glob
from fnmatch import fnmatchcase
try:  # Import either the C or pure-Python YAML parser
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader
import yaml

# Regex for type names in the YAML file. Optional *nnn indicates array.
TYPE_RE = re.compile(r'[a-z_A-Z]\w*(:?\s*\*\s*\d+)?$')

# Regex for integer ranges A..B[..C]
INT_RANGE_RE = re.compile(r'\s*(-?\d+)\s*\.\.\s*(-?\d+)\s*(?:\.\.\s*(-?\d+)\s*)?$')

# Regex for include: YAML extension
INCLUDE_RE = re.compile(r'include\s*:\s*(.*)')

# Regex for complex types
COMPLEX_RE = re.compile(r'f\d+_c$')

args = {}
testcases = set()
datatypes = {}
param = {}


def main():
    args.update(parse_args().__dict__)
    for doc in get_yaml_docs():
        process_doc(doc)


def process_doc(doc):
    """Process one document in the YAML file"""

    # Ignore empty documents
    if not doc or not doc.get('Tests'):
        return

    # Clear datatypes and params from previous documents
    datatypes.clear()
    param.clear()

    # Return dictionary of all known datatypes
    datatypes.update(get_datatypes(doc))

    # Arguments structure corresponding to C/C++ structure
    param['Arguments'] = type('Arguments', (ctypes.Structure,),
                              {'_fields_': get_arguments(doc)})

    # Special names which get expanded as lists of arguments
    param['dict_lists_to_expand'] = doc.get('Dictionary lists to expand') or ()

    # Lists which are not expanded
    param['lists_to_not_expand'] = doc.get('Lists to not expand') or ()

    # Defaults
    defaults = doc.get('Defaults') or {}

    # Known Bugs
    param['known_bugs'] = doc.get('Known bugs') or []

    # Functions
    param['Functions'] = doc.get('Functions') or {}

    # Instantiate all of the tests, starting with defaults
    for test in doc['Tests']:
        case = defaults.copy()
        case.update(test)
        generate(case, instantiate)


def parse_args():
    """Parse command-line arguments, returning input and output files"""
    parser = argparse.ArgumentParser(description="""
Expand rocSPARSE YAML test data file into binary Arguments records
""")
    parser.add_argument('infile',
                        nargs='?',
                        type=argparse.FileType('r'),
                        default=sys.stdin)
    parser.add_argument('-o', '--out',
                        dest='outfile',
                        type=argparse.FileType('wb'),
                        default=sys.stdout)
    parser.add_argument('-I',
                        help="Add include path",
                        action='append',
                        dest='includes',
                        default=[])
    parser.add_argument('-t', '--template',
                        type=argparse.FileType('r'))
    parser.add_argument('-m', '--matrices-dir',
                        dest='matrices-dir',
                        default='./')
    return parser.parse_args()


def read_yaml_file(file):
    """Read the YAML file, processing include: lines as an extension"""
    file_dir = os.path.dirname(file.name) or os.getcwd()
    source = []
    for line_no, line in enumerate(file, start=1):
        # Keep track of file names and line numbers for each line of YAML
        match = line.startswith('include') and INCLUDE_RE.match(line)
        if not match:
            source.append([line, file.name, line_no])
        else:
            include_file = match.group(1)
            include_dirs = [file_dir] + args['includes']
            for path in include_dirs:
                path = os.path.join(path, include_file)
                if os.path.exists(path):
                    source.extend(read_yaml_file(open(path, 'r')))
                    break
            else:
                sys.exit("In file " + file.name + ", line " +
                         str(line_no) + ", column " + str(match.start(1)+1) +
                         ":\n" + line.rstrip() + "\n" + " " * match.start(1) +
                         "^\nCannot open " + include_file +
                         "\n\nInclude paths:\n" + "\n".join(include_dirs))
    file.close()
    return source


def get_yaml_docs():
    """Parse the YAML file"""
    source = read_yaml_file(args['infile'])

    if args.get('template'):
        source = read_yaml_file(args['template']) + source

    source_str = ''.join([line[0] for line in source])

    def mark_str(mark):
        line = source[mark.line]
        return("In file " + line[1] + ", line " + str(line[2]) + ", column " +
               str(mark.column + 1) + ":\n" + line[0].rstrip() + "\n" +
               ' ' * mark.column + "^\n")

    # We iterate through all of the documents to properly diagnose errors,
    # because the load_all generator does not handle exceptions correctly.
    docs = []
    load = Loader(source_str)
    while load.check_data():
        try:
            doc = load.get_data()
        except yaml.YAMLError as err:
            sys.exit((mark_str(err.problem_mark) if err.problem_mark else "") +
                     (err.problem + "\n" if err.problem else "") +
                     (err.note + "\n" if err.note else ""))
        else:
            docs.append(doc)
    return docs


def get_datatypes(doc):
    """ Get datatypes from YAML doc"""
    dt = ctypes.__dict__.copy()
    for declaration in doc.get('Datatypes') or ():
        for name, decl in declaration.items():
            if isinstance(decl, dict):
                # Create derived class type based on bases and attr entries
                dt[name] = type(name,
                                tuple([eval(t, dt)
                                       for t in decl.get('bases') or ()
                                       if TYPE_RE.match(t)]
                                      ), decl.get('attr') or {})
                # Import class' attributes into the datatype namespace
                for subtype in decl.get('attr') or {}:
                    if TYPE_RE.match(subtype):
                        dt[subtype] = eval(name+'.'+subtype, dt)
            elif isinstance(decl, str) and TYPE_RE.match(decl):
                dt[name] = dt[decl]
            else:
                sys.exit("Unrecognized data type "+name+": "+repr(decl))
    return dt


def get_arguments(doc):
    """The kernel argument list, with argument names and types"""
    return [(var, eval(decl[var], datatypes))
            for decl in doc.get('Arguments') or ()
            if len(decl) == 1
            for var in decl
            if TYPE_RE.match(decl[var])]


def setdefaults(test):
    """Set default values for parameters"""
    # Do not put constant defaults here -- use rocsparse_common.yaml for that.
    # These are only for dynamic defaults
    # TODO: This should be ideally moved to YAML file, with eval'd expressions.

    if test['transA'] == 111 or test['transB'] == 111:
        test.setdefault('lda', 0)
        test.setdefault('ldb', 0)
        test.setdefault('ldc', 0)
        test.setdefault('ldd', 0)
    else:
        test.setdefault('lda', test['M'] if test['transA'] == 111 else
                        test['K'])
        test.setdefault('ldb', test['K'] if test['transB'] == 111 else
                        test['N'])
        test.setdefault('ldc', test['M'])
        test.setdefault('ldd', test['M'])



def write_signature(out):
    """Write the signature used to verify binary file compatibility"""
    if 'signature_written' not in args:
        sig = 0
        byt = bytearray("rocSPARSE", 'utf_8')
        byt.append(0)
        last_ofs = 0
        for (name, ctype) in param['Arguments']._fields_:
            member = getattr(param['Arguments'], name)
            for i in range(0, member.offset - last_ofs):
                byt.append(0)
            for i in range(0, member.size):
                byt.append(sig ^ i)
            sig = (sig + 89) % 256
            last_ofs = member.offset + member.size
        for i in range(0, ctypes.sizeof(param['Arguments']) - last_ofs):
            byt.append(0)
        byt.extend(bytes("ROCsparse", 'utf_8'))
        byt.append(0)
        out.write(byt)
        args['signature_written'] = True


def write_test(test):
    """Write the test case out to the binary file if not seen already"""

    # For each argument declared in arguments, we generate a positional
    # argument in the Arguments constructor. For strings, we pass the
    # value of the string directly. For arrays, we unpack their contents
    # into the ctype array constructor and pass the ctype array. For
    # scalars, we coerce the string/numeric value into ctype.
    arg = []
    for name, ctype in param['Arguments']._fields_:
        try:
            if issubclass(ctype, ctypes.Array):
                if issubclass(ctype._type_, ctypes.c_char):
                    arg.append(bytes(test[name], 'utf_8'))
                else:
                    arg.append(ctype(*test[name]))
            elif issubclass(ctype, ctypes.c_char):
                arg.append(bytes(test[name], 'utf_8'))
            else:
                arg.append(ctype(test[name]))
        except TypeError as err:
            sys.exit("TypeError: " + str(err) + " for " + name +
                     ", which has type " + str(type(test[name])) + "\n")

    byt = bytes(param['Arguments'](*arg))
    if byt not in testcases:
        testcases.add(byt)
        write_signature(args['outfile'])
        args['outfile'].write(byt)


def instantiate(test):
    """Instantiate a given test case"""
    test = test.copy()

    # Any Arguments fields declared as enums
    enum_args = [decl[0] for decl in param['Arguments']._fields_
                 if decl[1].__module__ == '__main__']
    try:
        setdefaults(test)

        # If no enum arguments are complex, clear alphai and betai
        for typename in enum_args:
            if COMPLEX_RE.match(test[typename]):
                break
        else:
            for name in ('alphai', 'betai', 'boostvali'):
                if name in test:
                    test[name] = 0.0

        # For enum arguments, replace name with value
        for typename in enum_args:
            test[typename] = datatypes[test[typename]]

        # Match known bugs
        if test['category'] not in ('known_bug', 'disabled'):
            for bug in param['known_bugs']:
                for key, value in bug.items():
                    if key not in test:
                        break
                    if key == 'function':
                        if not fnmatchcase(test[key], value):
                            break
                    # For keys declared as enums, compare resulting values
                    elif test[key] != (datatypes.get(value) if key in enum_args
                                       else value):
                        break
                else:  # All values specified in known bug match test case
                    test['category'] = 'known_bug'
                    break

        write_test(test)

    except KeyError as err:
        sys.exit("Undefined value " + str(err) + "\n" + str(test))


def generate(test, function):
    """Generate test combinations by iterating across lists recursively"""
    test = test.copy()

    # For specially named lists, they are expanded and merged into the test
    # argument list. When the list name is a dictionary of length 1, its pairs
    # indicate that the argument named by its key takes on values paired with
    # the argument named by its value, which is another dictionary list. We
    # process the value dictionaries' keys in alphabetic order, to ensure
    # deterministic test ordering.
    for argname in param['dict_lists_to_expand']:
        if type(argname) == dict:
            if len(argname) == 1:
                arg, target = list(argname.items())[0]
                if arg in test and type(test[arg]) == dict:
                    pairs = sorted(list(test[arg].items()), key=lambda x: x[0])
                    for test[arg], test[target] in pairs:
                        generate(test, function)
                    return
        elif argname in test and type(test[argname]) in (tuple, list, dict):
            # Pop the list and iterate across it
            ilist = test.pop(argname)

            # For a bare dictionary, wrap it in a list and apply it once
            for item in [ilist] if type(ilist) == dict else ilist:
                try:
                    case = test.copy()
                    case.update(item)  # original test merged with each item
                    generate(case, function)
                except TypeError as err:
                    sys.exit("TypeError: " + str(err) + " for " + argname +
                             ", which has type " + str(type(item)) +
                             "\nA name listed in \"Dictionary lists to expand\" "
                             "must be a defined as a dictionary.\n")
            return

    matrices_dir = str(args['matrices-dir'])

    if matrices_dir[-1] != "/":
        matrices_dir+="/"
    for key in sorted(list(test)):
        # Integer arguments which are ranges (A..B[..C]) are expanded
        if type(test[key]) == str:
            match = INT_RANGE_RE.match(str(test[key]))
            if match:
                for test[key] in range(int(match.group(1)),
                                       int(match.group(2))+1,
                                       int(match.group(3) or 1)):
                    generate(test, function)
                return

        # For sequence arguments, they are expanded into scalars
        elif (type(test[key]) in (tuple, list) and
              key not in param['lists_to_not_expand']):
            if key == "filename" and test[key] != "*":
                cleanlist=[]
                for test[key] in test[key]:
                    #
                    # Get argument.
                    #
                    filename_arg = matrices_dir + str(test[key])

                    #
                    # Check if this is a valid argument
                    #
                    if ((not os.path.isdir(filename_arg))
                        and (not glob.glob(filename_arg))
                        and (not glob.glob(filename_arg + ".csr"))
                        and (not glob.glob(filename_arg + ".bsr"))):
                        print("skip unrecognized filename, directory or filename glob expression: '" + test[key] + "'")
                    else:
                        cleanlist.append(test[key])

                if not cleanlist:
                    return;
                test[key] = cleanlist
                for test[key] in test[key]:
                    #
                    # Get argument.
                    #
                    filename_arg = matrices_dir + str(test[key])

                    #
                    # It is a directory.
                    #
                    if os.path.isdir(filename_arg):
                        #
                        # List the files.
                        #
                        names = glob.glob(filename_arg + "/*")
                        for name in names:
                            subpath=os.path.splitext(name.replace(matrices_dir,""))[0]
                            test[key]=[subpath]
                            generate(test,function)

                    else:
                        #
                        # Might be a glob expression
                        #
                        names = glob.glob(filename_arg)
                        if not names:
                            names = glob.glob(filename_arg + ".csr")
                            if not names:
                                names = glob.glob(filename_arg + ".bsr")
                            generate(test,function)

                        else:
                            for name in names:
                                subpath=os.path.splitext(name.replace(matrices_dir,""))[0]
                                test[key]=[subpath]
                                generate(test,function)


            else:
                for test[key] in test[key]:
                    generate(test,function)
                return


    # Replace typed function names with generic functions and types
    if 'rocsparse_function' in test:
        func = test.pop('rocsparse_function')
        if func in param['Functions']:
            test.update(param['Functions'][func])
        else:
            test['function'] = func
        generate(test, function)
        return

    function(test)



if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################


import argparse
import subprocess
import os
import re # regexp package
import sys
import tempfile
import json

#
#
#
def export_ptsdata(ibasename, obasename,xargs, yargs, results,verbose = False):

    len_xargs = len(xargs)
    for iplot in range(len(yargs)):
        one_only = True
        yarg=yargs[iplot]
        if yarg != "":
            yarg="_" + yarg
        yarg = yarg.replace("=","")
        yarg = yarg.replace(",","_")
        filename=obasename + yarg + ".csv"
        print("//rocsparse-bench-ptsdata  - writing into file '" + filename + "'")
        datafile = open(filename, "w+")
        for ixarg  in range(len_xargs):
            isample = iplot * len_xargs + ixarg
            tg = results[isample]["timing"]
            tg_raw_legend = ','.join(tg["raw_legend"].split())
            tg_raw = ','.join(tg["raw_data"].split())
        
            if len(tg_raw_legend) != 0:
                if verbose:
                    print('//rocsparse-bench-ptsdata  -  write pts data file : \'' + obasename + '.csv\'')
                if one_only:
                    one_only = False
                    datafile.write("test_name , " + tg_raw_legend + ",time, time_low, time_high, flops, flops_low, flops_high, bandwidth, bandwidth_low, bandwidth_high\n")
                datafile.write(ibasename + ", " + tg_raw + ", " +
                               tg["time"][0] + ", " +
                               tg["time"][1] + ", " +
                               tg["time"][2] + ", " +
                               tg["flops"][0] + ", " +
                               tg["flops"][1] + ", " +
                               tg["flops"][2] + ", " +
                               tg["bandwidth"][0] + ", " +
                               tg["bandwidth"][1] + ", "+
                               tg["bandwidth"][2] + "\n")
    datafile.close()


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,description="Convert a rocsparse benchmark .json file to a csv file.")
    parser.add_argument('-o', '--obasename',    required=False, default = 'a')
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = "store_true")
    user_args, unknown_args = parser.parse_known_args()
    verbose=user_args.verbose
    obasename = user_args.obasename
    if len(unknown_args) > 1:
        print('expecting only one input file.')
    ibasename = os.path.basename(os.path.splitext(unknown_args[0])[0])
    with open(unknown_args[0],"r") as f:
        case=json.load(f)

    cmd = case['cmdline']
    xargs = case['xargs']
    yargs = case['yargs']
    results = case['results']
    num_samples = len(results)
    len_xargs = len(xargs)

    if verbose:
        print('//rocsparse-bench-ptsdata')
        print('//rocsparse-bench-ptsdata  - file : \'' + unknown_args[0] + '\'')

    export_ptsdata( ibasename, obasename, xargs,yargs, results, verbose)

if __name__ == "__main__":
    main()




#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################

import argparse
import subprocess
import os
import re # regexp package
import sys
import tempfile
import json
import xml.etree.ElementTree as ET

#
#
# GNUPLOT COMMANDS FOR HISTOGRAM
#
#
def simple_histogram(out,ofilename,title,indices,ifilename,x_range,y_label,col_index, titles):
    nplots = len(indices)
    out.write("reset\n")
    out.write("set grid\n")
    out.write("set style fill solid 0.2\n")
    out.write("set style data histograms\n")
    out.write("set term pdfcairo enhanced color font 'Helvetica,9'\n")
    out.write("set output \"" + ofilename + "\"\n")
    out.write("set termoption noenhanced\n")
    out.write("set tmargin 0\n")
    out.write("set bmargin 0\n")
    out.write("set ylabel \"" + y_label + "\"\n")
    out.write("set xrange [" + str(x_range[0]) + ":" + str(x_range[1]) + "]\n")
    out.write("set offsets 0.25, 0.25, 0, 0\n")
    out.write("set xtics rotate by -45\n")
    out.write("set boxwidth 0.5\n")
    out.write("set size ratio 0.35\n")
    out.write("set style fill noborder\n")
    out.write("set title '" + title +"'\n")
    out.write("plot '"+ifilename+"' index "+str(indices[0])+" using "+str(col_index)+ ":xtic(1) title '"+ titles[0] +"' with histogram")
    for i in range(1,nplots):
        out.write(",\\\n '' index "+str(indices[i])+" using "+str(col_index)+":xtic(1) title '"+titles[i]+"' with histogram")
    out.write("\n")



def histogram(out,ofilename,title,indices,ifilename,x_range,y_label,col_index,col_index_low,col_index_high, titles, linear):
    nplots = len(indices)
    out.write("reset\n")
    out.write("set grid\n")
    out.write("set style fill solid 0.2\n")
    out.write("set style data histograms\n")
    out.write("set term pdfcairo enhanced color font 'Helvetica,9'\n")
    out.write("set output \"" + ofilename + "\"\n")
    out.write("set termoption noenhanced\n")
    out.write("set tmargin 0\n")
    out.write("set bmargin 0\n")

    # ERROR BARS
    out.write("set style histogram errorbars linewidth 1\n")
    out.write("set errorbars linecolor black\n")
    out.write("set bars front\n")

    out.write("set ylabel \"" + y_label + "\"\n")
    out.write("set xrange [" + str(x_range[0]) + ":" + str(x_range[1]) + "]\n")
    if not linear:
        out.write("set logscale y\n")
    out.write("set offsets 0.25, 0.25, 0, 0\n")
    out.write("set xtics rotate by -45\n")
    out.write("set boxwidth 0.5\n")
    out.write("set size ratio 0.35\n")
    out.write("set style fill noborder\n")
    out.write("set title '" + title +"'\n")
    out.write("plot '"+ifilename+"' index "+str(indices[0])+" using "+str(col_index)+":"+str(col_index_low)+":"+str(col_index_high)+":xtic(1) title '"+ titles[0] +"' with histogram")
    for i in range(1,nplots):
        out.write(",\\\n '' index "+str(indices[i])+" using "+str(col_index)+":"+str(col_index_low)+":"+str(col_index_high)+":xtic(1) title '"+titles[i]+"' with histogram")
    out.write("\n")


def curve(out,ofilename,title,indices,ifilename,x_range,y_label,col_index, titles):
    nplots = len(indices)
    out.write("reset\n")
    out.write("set grid\n")
    out.write("set term pdfcairo enhanced color font 'Helvetica,9'\n")
    out.write("set output \"" + ofilename + "\"\n")
    out.write("set termoption noenhanced\n")
    out.write("set tmargin 0\n")
    out.write("set bmargin 0\n")

    out.write("set ylabel \"" + y_label + "\"\n")
    out.write("set xrange [" + str(x_range[0]) + ":" + str(x_range[1]) + "]\n")
#    out.write("set yrange [1e-3:*]\n")
    out.write("set logscale y\n")
    out.write("set offsets 0.25, 0.25, 0, 0\n")
    out.write("set xtics rotate by -45\n")
    out.write("set size ratio 0.35\n")
    out.write("set style fill noborder\n")
    out.write("set title '" + title +"'\n")
    out.write("plot '"+ifilename+"' index "+str(indices[0])+" using "+str(col_index)+" with linespoints title '"+ titles[0] +"'")
    for i in range(1,nplots):
        out.write(",\\\n '' index "+str(indices[i])+" using "+str(col_index) + " with linespoints title '"+titles[i]+"'")
    out.write("\n")


def call(ifilename):
    cmdgnuplot = ["gnuplot", ifilename]
    proc = subprocess.Popen(cmdgnuplot)
    proc.wait()
    rc = proc.returncode
    if rc != 0:
        print('//rocsparse_bench_gnuplot_helper::call failed (err='+str(rc)+')')
        print('//rocsparse_bench_gnuplot_helper::note: check files \''+ifilename+'\'')
        exit(1)



#!/usr/bin/env python3

# ########################################################################
# Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################


import argparse
import subprocess
import os
import re # regexp package
import sys
import json
import glob
def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,description="Execute a set of rocsparse-bench commands from a .json file", epilog="The .json file must contain an array 'cmdlines' of strings, where each string contains the list of options to pass to rocsparse-bench to execute a test.\nExample:\n {\n   \"cmdlines\": [\"-f csrmv\",\n                \"-f csrmm\"]\n }\n")
    parser.add_argument('-w', '--workingdir',     required=False, default = './')
    parser.add_argument('-v', '--verbose',         required=False, default = False, action = "store_true")
    parser.add_argument('-d', '--debug',         required=False, default = False, action = "store_true")

    user_args, unknown_args = parser.parse_known_args()
    verbose=user_args.verbose
    workingdir = user_args.workingdir
    debug=user_args.debug
    datadir=os.getenv('ROCSPARSE_BENCH_DATA_DIR')
    if datadir == None:
        print('//rocsparse-bench-execute:error You must define environment variable ROCSPARSE_BENCH_DATA_DIR as the directory of sparse matrices.')
        print('//rocsparse-bench-execute:error   export ROCSPARSE_BENCH_DATA_DIR=<where-to-find-sparse-matrices>')
        exit(1)
    if verbose:
        print('//rocsparse-bench-execute:ROCSPARSE_BENCH_DATA_DIR ' + datadir)

    if len(unknown_args) > 1:
        print('expecting only one input file.')
    with open(unknown_args[0],"r") as f:
        case=json.load(f)
    cmdlines = case['cmdlines']
    num_cmdlines = len(cmdlines)
    progname = "rocsparse-bench"
    prog = os.path.join(workingdir, progname)
    if not os.path.isfile(prog):
        print("**** Error: unable to find " + prog)
        sys.exit(1)

    for i in range(num_cmdlines):
        # execute the cmdline
        full_cmd = prog + " " + cmdlines[i];
        print('//rocsparse-bench-execute:verbose:execute command "' + full_cmd+ '"')
        full_cmd=full_cmd.split(' ')
        subprocess_arg=[]
        for w in full_cmd:
            w=w.replace('${ROCSPARSE_BENCH_DATA_DIR}',datadir)
            gw=glob.glob(w)
            if (len(gw)!=0):
                for g in gw:
                    subprocess_arg.append(g)
            else:
                subprocess_arg.append(w)

        if verbose:
            print('//rocsparse-bench-execute:verbose:execute command with glob "' + ' '.join(subprocess_arg) + '"')
        proc = subprocess.Popen(subprocess_arg)
        proc.wait()
        rc = proc.returncode
        if rc != 0:
            print('//rocsparse-bench-execute:failure (err='+str(rc)+')')
            exit(1)

if __name__ == "__main__":
    main()




#!/usr/bin/python3

# ########################################################################
# Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ########################################################################
#
# Author: rocsparse-maintainer@amd.com
#
"""Copyright 2021 Advanced Micro Devices, Inc. All rights Reserved.
Run tests on build"""

import re
import os
import sys
import subprocess
import shlex
import argparse
import pathlib
import platform
from genericpath import exists
from fnmatch import fnmatchcase
from xml.dom import minidom
import multiprocessing
import time

args = {}
OS_info = {}

timeout = False
test_proc = None
stop = 0

test_script = [ 'cd %IDIR%', '%XML%' ]

def parse_args():
    """Parse command-line arguments"""
    parser = argparse.ArgumentParser(description="""
    Checks build arguments
    """)

    # Mutually exclusive group for --test and --emulation
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-t', '--test',
                       help='Test set to run from rtest.xml (required, e.g. osdb)')
    group.add_argument(      '--emulation', type=str, choices=['smoke', 'regression', 'extended'],
                        help='Enable specific emulation test mode, e.g. smoke test')

    parser.add_argument('-g', '--debug', required=False, default=False,  action='store_true',
                        help='Test Debug build (optional, default: false)')
    parser.add_argument('-o', '--output', type=str, required=False, default="xml",
                        help='Test output file (optional, default: test_detail.xml)')
    parser.add_argument(      '--install_dir', type=str, required=False, default="build",
                        help='Installation directory where build or release folders are (optional, default: build)')
    parser.add_argument(      '--fail_test', default=False, required=False, action='store_true',
                        help='Return as if test failed (optional, default: false)')
    # parser.add_argument('-v', '--verbose', required=False, default = False, action='store_true',
    #                     help='Verbose install (optional, default: False)')

    args = parser.parse_args()

    # Treat --emulation=xxx the same as -t xxx
    emulation_to_test = {
        'smoke': 'smoke',
        'regression': 'regression',
        'extended': 'extended'
    }

    if args.emulation in emulation_to_test:
        args.test = emulation_to_test[args.emulation]

    return args


def vram_detect():
    global OS_info
    OS_info["VRAM"] = 0
    if os.name == "nt":
        cmd = "hipinfo.exe"
        process = subprocess.run([cmd], stdout=subprocess.PIPE)
        for line_in in process.stdout.decode().splitlines():
            if 'totalGlobalMem' in line_in:
                OS_info["VRAM"] = float(line_in.split()[1])
                break
    else:
        cmd = "rocminfo"
        process = subprocess.run([cmd], stdout=subprocess.PIPE)
        for line_in in process.stdout.decode().splitlines():
            match = re.search(r'.*Size:.*([0-9]+)\(.*\).*KB', line_in, re.IGNORECASE)
            if match:
                OS_info["VRAM"] = float(match.group(1))/(1024*1024)
                break

def os_detect():
    global OS_info
    if os.name == "nt":
        OS_info["ID"] = platform.system()
    else:
        inf_file = "/etc/os-release"
        if os.path.exists(inf_file):
            with open(inf_file) as f:
                for line in f:
                    if "=" in line:
                        k,v = line.strip().split("=")
                        OS_info[k] = v.replace('"','')
    OS_info["NUM_PROC"] = os.cpu_count()
    vram_detect()
    print(OS_info)


def create_dir(dir_path):
    if os.path.isabs(dir_path):
        full_path = dir_path
    else:
        full_path = os.path.join( os.getcwd(), dir_path )
    return pathlib.Path(full_path).mkdir(parents=True, exist_ok=True)

def delete_dir(dir_path) :
    if (not os.path.exists(dir_path)):
        return
    if os.name == "nt":
        return run_cmd( "RMDIR" , f"/S /Q {dir_path}")
    else:
        linux_path = pathlib.Path(dir_path).absolute()
        return run_cmd( "rm" , f"-rf {linux_path}")

class TimerProcess(multiprocessing.Process):

    def __init__(self, start, stop, kill_pid):
        multiprocessing.Process.__init__(self)
        self.quit = multiprocessing.Event()
        self.timed_out = multiprocessing.Event()
        self.start_time = start
        self.max_time = stop
        self.kill_pid = kill_pid

    def run(self):
        while not self.quit.is_set():
            #print( f'time_stop {self.start_time} limit {self.max_time}')
            if (self.max_time == 0):
                return
            t = time.monotonic()
            if ( t - self.start_time > self.max_time ):
                print( f'killing {self.kill_pid} t {t}')
                if os.name == "nt":
                    cmd = ['TASKKILL', '/F', '/T', '/PID', str(self.kill_pid)]
                    proc = subprocess.Popen(cmd, stdout=sys.stdout, stderr=sys.stderr)
                else:
                    os.kill(self.kill_pid, signal.SIGKILL)
                self.timed_out.set()
                self.stop()
            pass

    def stop(self):
        self.quit.set()

    def stopped(self):
        return self.timed_out.is_set()


def time_stop(start, pid):
    global timeout, stop
    while (True):
        print( f'time_stop {start} limit {stop}')
        t = time.monotonic()
        if (stop == 0):
            return
        if ( (stop > 0) and (t - start > stop) ):
            print( f'killing {pid} t {t}')
            if os.name == "nt":
                cmd = ['TASKKILL', '/F', '/T', '/PID', str(pid)]
                proc = subprocess.Popen(cmd, stdout=sys.stdout, stderr=sys.stderr)
            else:
                test_proc.kill()
            timeout = True
            stop = 0
        time.sleep(0)

def run_cmd(cmd, test = False, time_limit = 0):
    global args
    global test_proc, timer_thread
    global stop
    if (cmd.startswith('cd ')):
        return os.chdir(cmd[3:])
    if (cmd.startswith('mkdir ')):
        return create_dir(cmd[6:])
    cmdline = f"{cmd}"
    print(cmdline)
    try:
        if not test:
            proc = subprocess.run(cmdline, check=True, stderr=subprocess.STDOUT, shell=True)
            status = proc.returncode
        else:
            sub_env = os.environ.copy()
            sub_env["PATH"] = os.getcwd() + os.pathsep + sub_env["PATH"]
            error = False
            timeout = False
            test_proc = subprocess.Popen(cmdline, text=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True, env=sub_env)
            if time_limit > 0:
                start = time.monotonic()
                #p = multiprocessing.Process(target=time_stop, args=(start, test_proc.pid))
                p = TimerProcess(start, time_limit, test_proc.pid)
                p.start()
            while True:
                output = test_proc.stdout.readline()
                if output == '' and test_proc.poll() is not None:
                    break
                elif output:
                    outstring = output.strip()
                    print (outstring)
                    error = error or re.search(r'error|fail', outstring, re.IGNORECASE)
            status = test_proc.poll()
            if time_limit > 0:
                p.stop()
                p.join()
                timeout = p.stopped()
                print(f"timeout {timeout}")
            if error:
                status = 1
            elif timeout:
                status = 2
            else:
                status = test_proc.returncode
    except:
        import traceback
        exc = traceback.format_exc()
        print( "Python Exception: {0}".format(exc) )
        status = 3
    return status

def batch(script, xml):
    global OS_info
    global args
    #
    cwd = pathlib.os.curdir
    rtest_cwd_path = os.path.abspath( os.path.join( cwd, 'rtest.xml') )
    if os.path.isfile(rtest_cwd_path) and os.path.dirname(rtest_cwd_path).endswith( "staging" ):
        # if in a staging directory then test locally
        test_dir = cwd
    else:
        if args.debug: build_type = "debug"
        else: build_type = "release"
        test_dir = f"{args.install_dir}/{build_type}/clients/staging"
    fail = False
    for i in range(len(script)):
        cmdline = script[i]
        xcmd = cmdline.replace('%IDIR%', test_dir)
        cmd = xcmd.replace('%ODIR%', args.output)
        if cmd.startswith('tdir '):
            if pathlib.Path(cmd[5:]).exists():
                return 0 # all further cmds skipped
            else:
                continue
        error = False
        if cmd.startswith('%XML%'):
            # run the matching tests listed in the xml test file
            var_subs = {}
            for var in xml.getElementsByTagName('var'):
                name = var.getAttribute('name')
                val = var.getAttribute('value')
                var_subs[name] = val
            for test in xml.getElementsByTagName('test'):
                sets = test.getAttribute('sets')
                runset = sets.split(',')
                if args.test in runset:
                    for run in test.getElementsByTagName('run'):
                        name = run.getAttribute('name')
                        vram_limit = run.getAttribute('vram_min')
                        if vram_limit:
                            if OS_info["VRAM"] < float(vram_limit):
                                print( f'***\n*** Skipped: {name} due to VRAM req.\n***')
                                continue
                        if name:
                            print( f'***\n*** Running: {name}\n***')
                        time_limit = run.getAttribute('time_max')
                        if time_limit:
                            timeout = float(time_limit)
                        else:
                            timeout = 0

                        raw_cmd = run.firstChild.data
                        var_cmd = raw_cmd.format_map(var_subs)
                        error = run_cmd(var_cmd, True, timeout)
                        if (error == 2):
                            print( f'***\n*** Timed out when running: {name}\n***')
        else:
            error = run_cmd(cmd)
        fail = fail or error

    if (fail):
        if (cmd == "%XML%"):
            print(f"FAILED xml test suite!")
        else:
            print(f"ERROR running: {cmd}")
        if (os.curdir != cwd):
            os.chdir( cwd )
        return 1
    if (os.curdir != cwd):
        os.chdir( cwd )
    return 0

def run_tests():
    global test_script
    global xmlDoc

    # install
    cwd = os.curdir

    xmlPath = os.path.join( cwd, 'rtest.xml')
    xmlDoc = minidom.parse( xmlPath )

    scripts = []
    scripts.append( test_script )
    for i in scripts:
        if (batch(i, xmlDoc)):
            #print("Failure in script. ABORTING")
            if (os.curdir != cwd):
                os.chdir( cwd )
            return 1
    if (os.curdir != cwd):
        os.chdir( cwd )
    return 0

def main():
    global args
    global timer_thread

    os_detect()
    args = parse_args()

    status = run_tests()

    if args.fail_test: status = 1

    if (status):
        sys.exit(status)

if __name__ == '__main__':
    main()
